# Copyright (C) 2022 CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

# CVAT REST API
#
# REST API for Computer Vision Annotation Tool (CVAT)  # noqa: E501
#
# The version of the OpenAPI document: 2.40.1
# Contact: support@cvat.ai
# Generated by: https://openapi-generator.tech


from __future__ import annotations

import typing

import re  # noqa: F401
import sys  # noqa: F401

from cvat_sdk.api_client.model_utils import (  # noqa: F401
    ApiTypeError,
    IModelData,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
    OpenApiModel
)
from cvat_sdk.api_client.exceptions import ApiAttributeError

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    # Enable introspection. Can't work normally due to cyclic imports
    from cvat_sdk.api_client.apis import *
    from cvat_sdk.api_client.models import *


def lazy_import():
    from cvat_sdk.api_client.model.quality_point_size_base import QualityPointSizeBase
    from cvat_sdk.api_client.model.quality_target_metric import QualityTargetMetric
    globals()['QualityPointSizeBase'] = QualityPointSizeBase
    globals()['QualityTargetMetric'] = QualityTargetMetric



class IQualitySettings(IModelData):
    """
    NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    # member type declarations
    id: int # noqa: E501
    """
    [optional]
    """

    task_id: typing.Union[int, none_type] # noqa: E501
    """
    [optional]
    """

    project_id: typing.Union[int, none_type] # noqa: E501
    """
    [optional]
    """

    job_filter: str # noqa: E501
    """
    [optional]
    A JSON-based logic expression used to filter jobs for quality validation. The filter supports various terms to specify conditions on job: ['assignee', 'id', 'stage', 'state', 'task_id', 'task_name', 'type'] .
    """

    inherit: bool # noqa: E501
    """
    [optional]
    Allow using project settings when computing task quality. Only applicable to task quality settings inside projects .
    """

    target_metric: typing.Union[typing.Any, none_type] # noqa: E501
    """
    [optional]
    The primary metric used for quality estimation  * `accuracy` - ACCURACY * `precision` - PRECISION * `recall` - RECALL.
    """

    target_metric_threshold: float # noqa: E501
    """
    [optional]
    Defines the minimal quality requirements in terms of the selected target metric. .
    """

    max_validations_per_job: int # noqa: E501
    """
    [optional]
    The maximum number of job validation attempts for the job assignee. The job can be automatically accepted if the job quality is above the required threshold, defined by the target threshold parameter. .
    """

    iou_threshold: float # noqa: E501
    """
    [optional, default: 0.4]
    Used for distinction between matched / unmatched shapes.
    """

    oks_sigma: float # noqa: E501
    """
    [optional, default: 0.09]
    Like IoU threshold, but for points. The percent of the bbox side, used as the radius of the circle around the GT point, where the checked point is expected to be. For boxes with different width and height, the \"side\" is computed as a geometric mean of the width and height. Read more: https://cocodataset.org/#keypoints-eval .
    """

    point_size_base: typing.Union[typing.Any, none_type] # noqa: E501
    """
    [optional]
    When comparing point annotations (including both separate points and point groups), the OKS sigma parameter defines matching area for each GT point based to the object size. The point size base parameter allows to configure how to determine the object size. If image_size, the image size is used. Useful if each point annotation represents a separate object or boxes grouped with points do not represent object boundaries. If group_bbox_size, the object size is based on the point group bbox size. Useful if each point group represents an object or there is a bbox grouped with points, representing the object size.   * `image_size` - IMAGE_SIZE * `group_bbox_size` - GROUP_BBOX_SIZE.
    """

    line_thickness: float # noqa: E501
    """
    [optional, default: 0.01]
    Thickness of polylines, relatively to the (image area) ^ 0.5. The distance to the boundary around the GT line, inside of which the checked line points should be .
    """

    low_overlap_threshold: float # noqa: E501
    """
    [optional, default: 0.8]
    Used for distinction between strong / weak (low_overlap) matches .
    """

    compare_line_orientation: bool # noqa: E501
    """
    [optional, default: True]
    Enables or disables polyline orientation comparison.
    """

    line_orientation_threshold: float # noqa: E501
    """
    [optional, default: 0.1]
    The minimal gain in the GT IoU between the given and reversed line directions to consider the line inverted. Only used when the 'compare_line_orientation' parameter is true .
    """

    compare_groups: bool # noqa: E501
    """
    [optional, default: True]
    Enables or disables annotation group checks.
    """

    group_match_threshold: float # noqa: E501
    """
    [optional, default: 0.5]
    Minimal IoU for groups to be considered matching. Only used when the 'compare_groups' parameter is true .
    """

    check_covered_annotations: bool # noqa: E501
    """
    [optional, default: True]
    Check for partially-covered annotations, useful in segmentation tasks .
    """

    object_visibility_threshold: float # noqa: E501
    """
    [optional, default: 0.05]
    Minimal visible area percent of the spatial annotations (polygons, masks) for reporting covered annotations. Only used when the 'object_visibility_threshold' parameter is true .
    """

    panoptic_comparison: bool # noqa: E501
    """
    [optional, default: True]
    Use only the visible part of the masks and polygons in comparisons .
    """

    compare_attributes: bool # noqa: E501
    """
    [optional, default: True]
    Enables or disables annotation attribute comparison.
    """

    empty_is_annotated: bool # noqa: E501
    """
    [optional, default: False]
    Consider empty frames annotated as \"empty\". This affects target metrics like accuracy in cases there are no annotations. If disabled, frames without annotations are counted as not matching (accuracy is 0). If enabled, accuracy will be 1 instead. This will also add virtual annotations to empty frames in the comparison results. .
    """

    created_date: datetime # noqa: E501
    """
    [optional]
    """

    updated_date: datetime # noqa: E501
    """
    [optional]
    """


class QualitySettings(ModelNormal, IQualitySettings):
    """
    NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      id (int): [optional]  # noqa: E501

      task_id (int, none_type): [optional]  # noqa: E501

      project_id (int, none_type): [optional]  # noqa: E501

      job_filter (str): A JSON-based logic expression used to filter jobs for quality validation. The filter supports various terms to specify conditions on job: ['assignee', 'id', 'stage', 'state', 'task_id', 'task_name', 'type'] . [optional]  # noqa: E501

      inherit (bool): Allow using project settings when computing task quality. Only applicable to task quality settings inside projects . [optional]  # noqa: E501

      target_metric (bool, date, datetime, dict, float, int, list, str, none_type): The primary metric used for quality estimation  * `accuracy` - ACCURACY * `precision` - PRECISION * `recall` - RECALL. [optional]  # noqa: E501

      target_metric_threshold (float): Defines the minimal quality requirements in terms of the selected target metric. . [optional]  # noqa: E501

      max_validations_per_job (int): The maximum number of job validation attempts for the job assignee. The job can be automatically accepted if the job quality is above the required threshold, defined by the target threshold parameter. . [optional]  # noqa: E501

      iou_threshold (float): Used for distinction between matched / unmatched shapes. [optional] if omitted the server will use the default value of 0.4  # noqa: E501

      oks_sigma (float): Like IoU threshold, but for points. The percent of the bbox side, used as the radius of the circle around the GT point, where the checked point is expected to be. For boxes with different width and height, the \"side\" is computed as a geometric mean of the width and height. Read more: https://cocodataset.org/#keypoints-eval . [optional] if omitted the server will use the default value of 0.09  # noqa: E501

      point_size_base (bool, date, datetime, dict, float, int, list, str, none_type): When comparing point annotations (including both separate points and point groups), the OKS sigma parameter defines matching area for each GT point based to the object size. The point size base parameter allows to configure how to determine the object size. If image_size, the image size is used. Useful if each point annotation represents a separate object or boxes grouped with points do not represent object boundaries. If group_bbox_size, the object size is based on the point group bbox size. Useful if each point group represents an object or there is a bbox grouped with points, representing the object size.   * `image_size` - IMAGE_SIZE * `group_bbox_size` - GROUP_BBOX_SIZE. [optional]  # noqa: E501

      line_thickness (float): Thickness of polylines, relatively to the (image area) ^ 0.5. The distance to the boundary around the GT line, inside of which the checked line points should be . [optional] if omitted the server will use the default value of 0.01  # noqa: E501

      low_overlap_threshold (float): Used for distinction between strong / weak (low_overlap) matches . [optional] if omitted the server will use the default value of 0.8  # noqa: E501

      compare_line_orientation (bool): Enables or disables polyline orientation comparison. [optional] if omitted the server will use the default value of True  # noqa: E501

      line_orientation_threshold (float): The minimal gain in the GT IoU between the given and reversed line directions to consider the line inverted. Only used when the 'compare_line_orientation' parameter is true . [optional] if omitted the server will use the default value of 0.1  # noqa: E501

      compare_groups (bool): Enables or disables annotation group checks. [optional] if omitted the server will use the default value of True  # noqa: E501

      group_match_threshold (float): Minimal IoU for groups to be considered matching. Only used when the 'compare_groups' parameter is true . [optional] if omitted the server will use the default value of 0.5  # noqa: E501

      check_covered_annotations (bool): Check for partially-covered annotations, useful in segmentation tasks . [optional] if omitted the server will use the default value of True  # noqa: E501

      object_visibility_threshold (float): Minimal visible area percent of the spatial annotations (polygons, masks) for reporting covered annotations. Only used when the 'object_visibility_threshold' parameter is true . [optional] if omitted the server will use the default value of 0.05  # noqa: E501

      panoptic_comparison (bool): Use only the visible part of the masks and polygons in comparisons . [optional] if omitted the server will use the default value of True  # noqa: E501

      compare_attributes (bool): Enables or disables annotation attribute comparison. [optional] if omitted the server will use the default value of True  # noqa: E501

      empty_is_annotated (bool): Consider empty frames annotated as \"empty\". This affects target metrics like accuracy in cases there are no annotations. If disabled, frames without annotations are counted as not matching (accuracy is 0). If enabled, accuracy will be 1 instead. This will also add virtual annotations to empty frames in the comparison results. . [optional] if omitted the server will use the default value of False  # noqa: E501

      created_date (datetime): [optional]  # noqa: E501

      updated_date (datetime): [optional]  # noqa: E501


      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.

    """

    allowed_values = {
    }

    validations = {
        ('job_filter',): {
            'max_length': 1024,
        },
        ('target_metric_threshold',): {
            'inclusive_maximum': 1,
            'inclusive_minimum': 0,
        },
        ('max_validations_per_job',): {
            'inclusive_maximum': 2147483647,
            'inclusive_minimum': 0,
        },
        ('iou_threshold',): {
            'inclusive_maximum': 1,
            'inclusive_minimum': 0,
        },
        ('oks_sigma',): {
            'inclusive_maximum': 1,
            'inclusive_minimum': 0,
        },
        ('line_thickness',): {
            'inclusive_maximum': 1,
            'inclusive_minimum': 0,
        },
        ('low_overlap_threshold',): {
            'inclusive_maximum': 1,
            'inclusive_minimum': 0,
        },
        ('line_orientation_threshold',): {
            'inclusive_maximum': 1,
            'inclusive_minimum': 0,
        },
        ('group_match_threshold',): {
            'inclusive_maximum': 1,
            'inclusive_minimum': 0,
        },
        ('object_visibility_threshold',): {
            'inclusive_maximum': 1,
            'inclusive_minimum': 0,
        },
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'id': (int,),  # noqa: E501
            'task_id': (int, none_type,),  # noqa: E501
            'project_id': (int, none_type,),  # noqa: E501
            'job_filter': (str,),  # noqa: E501
            'inherit': (bool,),  # noqa: E501
            'target_metric': (bool, date, datetime, dict, float, int, list, str, none_type,),  # noqa: E501
            'target_metric_threshold': (float,),  # noqa: E501
            'max_validations_per_job': (int,),  # noqa: E501
            'iou_threshold': (float,),  # noqa: E501
            'oks_sigma': (float,),  # noqa: E501
            'point_size_base': (bool, date, datetime, dict, float, int, list, str, none_type,),  # noqa: E501
            'line_thickness': (float,),  # noqa: E501
            'low_overlap_threshold': (float,),  # noqa: E501
            'compare_line_orientation': (bool,),  # noqa: E501
            'line_orientation_threshold': (float,),  # noqa: E501
            'compare_groups': (bool,),  # noqa: E501
            'group_match_threshold': (float,),  # noqa: E501
            'check_covered_annotations': (bool,),  # noqa: E501
            'object_visibility_threshold': (float,),  # noqa: E501
            'panoptic_comparison': (bool,),  # noqa: E501
            'compare_attributes': (bool,),  # noqa: E501
            'empty_is_annotated': (bool,),  # noqa: E501
            'created_date': (datetime,),  # noqa: E501
            'updated_date': (datetime,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None



    attribute_map = {
        'id': 'id',  # noqa: E501
        'task_id': 'task_id',  # noqa: E501
        'project_id': 'project_id',  # noqa: E501
        'job_filter': 'job_filter',  # noqa: E501
        'inherit': 'inherit',  # noqa: E501
        'target_metric': 'target_metric',  # noqa: E501
        'target_metric_threshold': 'target_metric_threshold',  # noqa: E501
        'max_validations_per_job': 'max_validations_per_job',  # noqa: E501
        'iou_threshold': 'iou_threshold',  # noqa: E501
        'oks_sigma': 'oks_sigma',  # noqa: E501
        'point_size_base': 'point_size_base',  # noqa: E501
        'line_thickness': 'line_thickness',  # noqa: E501
        'low_overlap_threshold': 'low_overlap_threshold',  # noqa: E501
        'compare_line_orientation': 'compare_line_orientation',  # noqa: E501
        'line_orientation_threshold': 'line_orientation_threshold',  # noqa: E501
        'compare_groups': 'compare_groups',  # noqa: E501
        'group_match_threshold': 'group_match_threshold',  # noqa: E501
        'check_covered_annotations': 'check_covered_annotations',  # noqa: E501
        'object_visibility_threshold': 'object_visibility_threshold',  # noqa: E501
        'panoptic_comparison': 'panoptic_comparison',  # noqa: E501
        'compare_attributes': 'compare_attributes',  # noqa: E501
        'empty_is_annotated': 'empty_is_annotated',  # noqa: E501
        'created_date': 'created_date',  # noqa: E501
        'updated_date': 'updated_date',  # noqa: E501
    }

    read_only_vars = {
        'id',  # noqa: E501
        'created_date',  # noqa: E501
        'updated_date',  # noqa: E501
    }

    _composed_schemas = {}

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, *args, **kwargs):  # noqa: E501
        """QualitySettings - a model defined in OpenAPI

        Keyword Args:
            id (int): [optional]  # noqa: E501

            task_id (int, none_type): [optional]  # noqa: E501

            project_id (int, none_type): [optional]  # noqa: E501

            job_filter (str): A JSON-based logic expression used to filter jobs for quality validation. The filter supports various terms to specify conditions on job: ['assignee', 'id', 'stage', 'state', 'task_id', 'task_name', 'type'] . [optional]  # noqa: E501

            inherit (bool): Allow using project settings when computing task quality. Only applicable to task quality settings inside projects . [optional]  # noqa: E501

            target_metric (bool, date, datetime, dict, float, int, list, str, none_type): The primary metric used for quality estimation  * `accuracy` - ACCURACY * `precision` - PRECISION * `recall` - RECALL. [optional]  # noqa: E501

            target_metric_threshold (float): Defines the minimal quality requirements in terms of the selected target metric. . [optional]  # noqa: E501

            max_validations_per_job (int): The maximum number of job validation attempts for the job assignee. The job can be automatically accepted if the job quality is above the required threshold, defined by the target threshold parameter. . [optional]  # noqa: E501

            iou_threshold (float): Used for distinction between matched / unmatched shapes. [optional] if omitted the server will use the default value of 0.4  # noqa: E501

            oks_sigma (float): Like IoU threshold, but for points. The percent of the bbox side, used as the radius of the circle around the GT point, where the checked point is expected to be. For boxes with different width and height, the \"side\" is computed as a geometric mean of the width and height. Read more: https://cocodataset.org/#keypoints-eval . [optional] if omitted the server will use the default value of 0.09  # noqa: E501

            point_size_base (bool, date, datetime, dict, float, int, list, str, none_type): When comparing point annotations (including both separate points and point groups), the OKS sigma parameter defines matching area for each GT point based to the object size. The point size base parameter allows to configure how to determine the object size. If image_size, the image size is used. Useful if each point annotation represents a separate object or boxes grouped with points do not represent object boundaries. If group_bbox_size, the object size is based on the point group bbox size. Useful if each point group represents an object or there is a bbox grouped with points, representing the object size.   * `image_size` - IMAGE_SIZE * `group_bbox_size` - GROUP_BBOX_SIZE. [optional]  # noqa: E501

            line_thickness (float): Thickness of polylines, relatively to the (image area) ^ 0.5. The distance to the boundary around the GT line, inside of which the checked line points should be . [optional] if omitted the server will use the default value of 0.01  # noqa: E501

            low_overlap_threshold (float): Used for distinction between strong / weak (low_overlap) matches . [optional] if omitted the server will use the default value of 0.8  # noqa: E501

            compare_line_orientation (bool): Enables or disables polyline orientation comparison. [optional] if omitted the server will use the default value of True  # noqa: E501

            line_orientation_threshold (float): The minimal gain in the GT IoU between the given and reversed line directions to consider the line inverted. Only used when the 'compare_line_orientation' parameter is true . [optional] if omitted the server will use the default value of 0.1  # noqa: E501

            compare_groups (bool): Enables or disables annotation group checks. [optional] if omitted the server will use the default value of True  # noqa: E501

            group_match_threshold (float): Minimal IoU for groups to be considered matching. Only used when the 'compare_groups' parameter is true . [optional] if omitted the server will use the default value of 0.5  # noqa: E501

            check_covered_annotations (bool): Check for partially-covered annotations, useful in segmentation tasks . [optional] if omitted the server will use the default value of True  # noqa: E501

            object_visibility_threshold (float): Minimal visible area percent of the spatial annotations (polygons, masks) for reporting covered annotations. Only used when the 'object_visibility_threshold' parameter is true . [optional] if omitted the server will use the default value of 0.05  # noqa: E501

            panoptic_comparison (bool): Use only the visible part of the masks and polygons in comparisons . [optional] if omitted the server will use the default value of True  # noqa: E501

            compare_attributes (bool): Enables or disables annotation attribute comparison. [optional] if omitted the server will use the default value of True  # noqa: E501

            empty_is_annotated (bool): Consider empty frames annotated as \"empty\". This affects target metrics like accuracy in cases there are no annotations. If disabled, frames without annotations are counted as not matching (accuracy is 0). If enabled, accuracy will be 1 instead. This will also add virtual annotations to empty frames in the comparison results. . [optional] if omitted the server will use the default value of False  # noqa: E501

            created_date (datetime): [optional]  # noqa: E501

            updated_date (datetime): [optional]  # noqa: E501

            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
"""

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', True)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            for arg in args:
                if isinstance(arg, dict):
                    kwargs.update(arg)
                else:
                    raise ApiTypeError(
                        "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                            args,
                            self.__class__.__name__,
                        ),
                        path_to_item=_path_to_item,
                        valid_classes=(self.__class__,),
                    )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
        return self

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, *args, **kwargs):  # noqa: E501
        """QualitySettings - a model defined in OpenAPI

        Keyword Args:
            id (int): [optional]  # noqa: E501

            task_id (int, none_type): [optional]  # noqa: E501

            project_id (int, none_type): [optional]  # noqa: E501

            job_filter (str): A JSON-based logic expression used to filter jobs for quality validation. The filter supports various terms to specify conditions on job: ['assignee', 'id', 'stage', 'state', 'task_id', 'task_name', 'type'] . [optional]  # noqa: E501

            inherit (bool): Allow using project settings when computing task quality. Only applicable to task quality settings inside projects . [optional]  # noqa: E501

            target_metric (bool, date, datetime, dict, float, int, list, str, none_type): The primary metric used for quality estimation  * `accuracy` - ACCURACY * `precision` - PRECISION * `recall` - RECALL. [optional]  # noqa: E501

            target_metric_threshold (float): Defines the minimal quality requirements in terms of the selected target metric. . [optional]  # noqa: E501

            max_validations_per_job (int): The maximum number of job validation attempts for the job assignee. The job can be automatically accepted if the job quality is above the required threshold, defined by the target threshold parameter. . [optional]  # noqa: E501

            iou_threshold (float): Used for distinction between matched / unmatched shapes. [optional] if omitted the server will use the default value of 0.4  # noqa: E501

            oks_sigma (float): Like IoU threshold, but for points. The percent of the bbox side, used as the radius of the circle around the GT point, where the checked point is expected to be. For boxes with different width and height, the \"side\" is computed as a geometric mean of the width and height. Read more: https://cocodataset.org/#keypoints-eval . [optional] if omitted the server will use the default value of 0.09  # noqa: E501

            point_size_base (bool, date, datetime, dict, float, int, list, str, none_type): When comparing point annotations (including both separate points and point groups), the OKS sigma parameter defines matching area for each GT point based to the object size. The point size base parameter allows to configure how to determine the object size. If image_size, the image size is used. Useful if each point annotation represents a separate object or boxes grouped with points do not represent object boundaries. If group_bbox_size, the object size is based on the point group bbox size. Useful if each point group represents an object or there is a bbox grouped with points, representing the object size.   * `image_size` - IMAGE_SIZE * `group_bbox_size` - GROUP_BBOX_SIZE. [optional]  # noqa: E501

            line_thickness (float): Thickness of polylines, relatively to the (image area) ^ 0.5. The distance to the boundary around the GT line, inside of which the checked line points should be . [optional] if omitted the server will use the default value of 0.01  # noqa: E501

            low_overlap_threshold (float): Used for distinction between strong / weak (low_overlap) matches . [optional] if omitted the server will use the default value of 0.8  # noqa: E501

            compare_line_orientation (bool): Enables or disables polyline orientation comparison. [optional] if omitted the server will use the default value of True  # noqa: E501

            line_orientation_threshold (float): The minimal gain in the GT IoU between the given and reversed line directions to consider the line inverted. Only used when the 'compare_line_orientation' parameter is true . [optional] if omitted the server will use the default value of 0.1  # noqa: E501

            compare_groups (bool): Enables or disables annotation group checks. [optional] if omitted the server will use the default value of True  # noqa: E501

            group_match_threshold (float): Minimal IoU for groups to be considered matching. Only used when the 'compare_groups' parameter is true . [optional] if omitted the server will use the default value of 0.5  # noqa: E501

            check_covered_annotations (bool): Check for partially-covered annotations, useful in segmentation tasks . [optional] if omitted the server will use the default value of True  # noqa: E501

            object_visibility_threshold (float): Minimal visible area percent of the spatial annotations (polygons, masks) for reporting covered annotations. Only used when the 'object_visibility_threshold' parameter is true . [optional] if omitted the server will use the default value of 0.05  # noqa: E501

            panoptic_comparison (bool): Use only the visible part of the masks and polygons in comparisons . [optional] if omitted the server will use the default value of True  # noqa: E501

            compare_attributes (bool): Enables or disables annotation attribute comparison. [optional] if omitted the server will use the default value of True  # noqa: E501

            empty_is_annotated (bool): Consider empty frames annotated as \"empty\". This affects target metrics like accuracy in cases there are no annotations. If disabled, frames without annotations are counted as not matching (accuracy is 0). If enabled, accuracy will be 1 instead. This will also add virtual annotations to empty frames in the comparison results. . [optional] if omitted the server will use the default value of False  # noqa: E501

            created_date (datetime): [optional]  # noqa: E501

            updated_date (datetime): [optional]  # noqa: E501

            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
"""

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            for arg in args:
                if isinstance(arg, dict):
                    kwargs.update(arg)
                else:
                    raise ApiTypeError(
                        "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                            args,
                            self.__class__.__name__,
                        ),
                        path_to_item=_path_to_item,
                        valid_classes=(self.__class__,),
                    )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
            if var_name in self.read_only_vars:
                raise ApiAttributeError(f"`{var_name}` is a read-only attribute. Use `from_openapi_data` to instantiate "
                                     f"class with read only attributes.")




# Copyright (C) 2022 CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

# CVAT REST API
#
# REST API for Computer Vision Annotation Tool (CVAT)  # noqa: E501
#
# The version of the OpenAPI document: 2.40.1
# Contact: support@cvat.ai
# Generated by: https://openapi-generator.tech


# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from from cvat_sdk.api_client.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from cvat_sdk.api_client.model.about import About, IAbout
from cvat_sdk.api_client.model.accept_invitation_read import AcceptInvitationRead, IAcceptInvitationRead
from cvat_sdk.api_client.model.annotation_conflict import AnnotationConflict, IAnnotationConflict
from cvat_sdk.api_client.model.annotation_conflict_annotation_type import AnnotationConflictAnnotationType, IAnnotationConflictAnnotationType
from cvat_sdk.api_client.model.annotation_conflict_severity import AnnotationConflictSeverity, IAnnotationConflictSeverity
from cvat_sdk.api_client.model.annotation_conflict_type import AnnotationConflictType, IAnnotationConflictType
from cvat_sdk.api_client.model.annotation_file_request import AnnotationFileRequest, IAnnotationFileRequest
from cvat_sdk.api_client.model.annotation_guide_read import AnnotationGuideRead, IAnnotationGuideRead
from cvat_sdk.api_client.model.annotation_guide_write_request import AnnotationGuideWriteRequest, IAnnotationGuideWriteRequest
from cvat_sdk.api_client.model.annotation_id import AnnotationId, IAnnotationId
from cvat_sdk.api_client.model.annotation_id_shape_type import AnnotationIdShapeType
from cvat_sdk.api_client.model.asset_read import AssetRead, IAssetRead
from cvat_sdk.api_client.model.attribute import Attribute, IAttribute
from cvat_sdk.api_client.model.attribute_request import AttributeRequest, IAttributeRequest
from cvat_sdk.api_client.model.attribute_val import AttributeVal, IAttributeVal
from cvat_sdk.api_client.model.attribute_val_request import AttributeValRequest, IAttributeValRequest
from cvat_sdk.api_client.model.basic_organization import BasicOrganization, IBasicOrganization
from cvat_sdk.api_client.model.basic_user import BasicUser, IBasicUser
from cvat_sdk.api_client.model.basic_user_request import BasicUserRequest, IBasicUserRequest
from cvat_sdk.api_client.model.chunk_type import ChunkType, IChunkType
from cvat_sdk.api_client.model.client_events import ClientEvents, IClientEvents
from cvat_sdk.api_client.model.client_events_request import ClientEventsRequest, IClientEventsRequest
from cvat_sdk.api_client.model.client_events_request_previous_event import ClientEventsRequestPreviousEvent
from cvat_sdk.api_client.model.cloud_storage_content import CloudStorageContent, ICloudStorageContent
from cvat_sdk.api_client.model.cloud_storage_read import CloudStorageRead, ICloudStorageRead
from cvat_sdk.api_client.model.cloud_storage_read_owner import CloudStorageReadOwner
from cvat_sdk.api_client.model.cloud_storage_write_request import CloudStorageWriteRequest, ICloudStorageWriteRequest
from cvat_sdk.api_client.model.comment_read import CommentRead, ICommentRead
from cvat_sdk.api_client.model.comment_write_request import CommentWriteRequest, ICommentWriteRequest
from cvat_sdk.api_client.model.comments_summary import CommentsSummary, ICommentsSummary
from cvat_sdk.api_client.model.consensus_merge_create_request import ConsensusMergeCreateRequest, IConsensusMergeCreateRequest
from cvat_sdk.api_client.model.consensus_settings import ConsensusSettings, IConsensusSettings
from cvat_sdk.api_client.model.credentials_type_enum import CredentialsTypeEnum, ICredentialsTypeEnum
from cvat_sdk.api_client.model.data_meta_read import DataMetaRead, IDataMetaRead
from cvat_sdk.api_client.model.data_request import DataRequest, IDataRequest
from cvat_sdk.api_client.model.data_request_validation_params import DataRequestValidationParams
from cvat_sdk.api_client.model.data_response import DataResponse
from cvat_sdk.api_client.model.dataset_file_request import DatasetFileRequest, IDatasetFileRequest
from cvat_sdk.api_client.model.dataset_format import DatasetFormat, IDatasetFormat
from cvat_sdk.api_client.model.dataset_formats import DatasetFormats, IDatasetFormats
from cvat_sdk.api_client.model.event import Event, IEvent
from cvat_sdk.api_client.model.event_request import EventRequest, IEventRequest
from cvat_sdk.api_client.model.events import Events, IEvents
from cvat_sdk.api_client.model.events_enum import EventsEnum, IEventsEnum
from cvat_sdk.api_client.model.file_info import FileInfo, IFileInfo
from cvat_sdk.api_client.model.file_info_type_enum import FileInfoTypeEnum, IFileInfoTypeEnum
from cvat_sdk.api_client.model.frame_meta import FrameMeta, IFrameMeta
from cvat_sdk.api_client.model.frame_selection_method import FrameSelectionMethod, IFrameSelectionMethod
from cvat_sdk.api_client.model.function_call import FunctionCall, IFunctionCall
from cvat_sdk.api_client.model.function_call_params import FunctionCallParams, IFunctionCallParams
from cvat_sdk.api_client.model.function_call_request import FunctionCallRequest, IFunctionCallRequest
from cvat_sdk.api_client.model.input_type_enum import InputTypeEnum, IInputTypeEnum
from cvat_sdk.api_client.model.invitation_read import InvitationRead, IInvitationRead
from cvat_sdk.api_client.model.invitation_write_request import InvitationWriteRequest, IInvitationWriteRequest
from cvat_sdk.api_client.model.issue_read import IssueRead, IIssueRead
from cvat_sdk.api_client.model.issue_write_request import IssueWriteRequest, IIssueWriteRequest
from cvat_sdk.api_client.model.issues_summary import IssuesSummary, IIssuesSummary
from cvat_sdk.api_client.model.job_read import JobRead, IJobRead
from cvat_sdk.api_client.model.job_read_assignee import JobReadAssignee
from cvat_sdk.api_client.model.job_read_target_storage import JobReadTargetStorage
from cvat_sdk.api_client.model.job_stage import JobStage, IJobStage
from cvat_sdk.api_client.model.job_status import JobStatus, IJobStatus
from cvat_sdk.api_client.model.job_type import JobType, IJobType
from cvat_sdk.api_client.model.job_validation_layout_read import JobValidationLayoutRead, IJobValidationLayoutRead
from cvat_sdk.api_client.model.job_write_request import JobWriteRequest, IJobWriteRequest
from cvat_sdk.api_client.model.jobs_summary import JobsSummary, IJobsSummary
from cvat_sdk.api_client.model.label import Label, ILabel
from cvat_sdk.api_client.model.label_mapping_entry_request import LabelMappingEntryRequest, ILabelMappingEntryRequest
from cvat_sdk.api_client.model.label_type import LabelType, ILabelType
from cvat_sdk.api_client.model.labeled_data import LabeledData, ILabeledData
from cvat_sdk.api_client.model.labeled_data_request import LabeledDataRequest, ILabeledDataRequest
from cvat_sdk.api_client.model.labeled_image import LabeledImage, ILabeledImage
from cvat_sdk.api_client.model.labeled_image_request import LabeledImageRequest, ILabeledImageRequest
from cvat_sdk.api_client.model.labeled_shape import LabeledShape, ILabeledShape
from cvat_sdk.api_client.model.labeled_shape_request import LabeledShapeRequest, ILabeledShapeRequest
from cvat_sdk.api_client.model.labeled_track import LabeledTrack, ILabeledTrack
from cvat_sdk.api_client.model.labeled_track_request import LabeledTrackRequest, ILabeledTrackRequest
from cvat_sdk.api_client.model.labels_summary import LabelsSummary, ILabelsSummary
from cvat_sdk.api_client.model.location_enum import LocationEnum, ILocationEnum
from cvat_sdk.api_client.model.login_serializer_ex_request import LoginSerializerExRequest, ILoginSerializerExRequest
from cvat_sdk.api_client.model.membership_read import MembershipRead, IMembershipRead
from cvat_sdk.api_client.model.meta_user import MetaUser
from cvat_sdk.api_client.model.null_enum import NullEnum, INullEnum
from cvat_sdk.api_client.model.online_function_call_request import OnlineFunctionCallRequest, IOnlineFunctionCallRequest
from cvat_sdk.api_client.model.operation_status import OperationStatus, IOperationStatus
from cvat_sdk.api_client.model.organization_read import OrganizationRead, IOrganizationRead
from cvat_sdk.api_client.model.organization_write_request import OrganizationWriteRequest, IOrganizationWriteRequest
from cvat_sdk.api_client.model.paginated_annotation_conflict_list import PaginatedAnnotationConflictList, IPaginatedAnnotationConflictList
from cvat_sdk.api_client.model.paginated_cloud_storage_read_list import PaginatedCloudStorageReadList, IPaginatedCloudStorageReadList
from cvat_sdk.api_client.model.paginated_comment_read_list import PaginatedCommentReadList, IPaginatedCommentReadList
from cvat_sdk.api_client.model.paginated_consensus_settings_list import PaginatedConsensusSettingsList, IPaginatedConsensusSettingsList
from cvat_sdk.api_client.model.paginated_invitation_read_list import PaginatedInvitationReadList, IPaginatedInvitationReadList
from cvat_sdk.api_client.model.paginated_issue_read_list import PaginatedIssueReadList, IPaginatedIssueReadList
from cvat_sdk.api_client.model.paginated_job_read_list import PaginatedJobReadList, IPaginatedJobReadList
from cvat_sdk.api_client.model.paginated_label_list import PaginatedLabelList, IPaginatedLabelList
from cvat_sdk.api_client.model.paginated_membership_read_list import PaginatedMembershipReadList, IPaginatedMembershipReadList
from cvat_sdk.api_client.model.paginated_meta_user_list import PaginatedMetaUserList, IPaginatedMetaUserList
from cvat_sdk.api_client.model.paginated_organization_read_list import PaginatedOrganizationReadList, IPaginatedOrganizationReadList
from cvat_sdk.api_client.model.paginated_project_read_list import PaginatedProjectReadList, IPaginatedProjectReadList
from cvat_sdk.api_client.model.paginated_quality_report_list import PaginatedQualityReportList, IPaginatedQualityReportList
from cvat_sdk.api_client.model.paginated_quality_settings_list import PaginatedQualitySettingsList, IPaginatedQualitySettingsList
from cvat_sdk.api_client.model.paginated_request_list import PaginatedRequestList, IPaginatedRequestList
from cvat_sdk.api_client.model.paginated_task_read_list import PaginatedTaskReadList, IPaginatedTaskReadList
from cvat_sdk.api_client.model.paginated_webhook_delivery_read_list import PaginatedWebhookDeliveryReadList, IPaginatedWebhookDeliveryReadList
from cvat_sdk.api_client.model.paginated_webhook_read_list import PaginatedWebhookReadList, IPaginatedWebhookReadList
from cvat_sdk.api_client.model.password_change_request import PasswordChangeRequest, IPasswordChangeRequest
from cvat_sdk.api_client.model.password_reset_confirm_request import PasswordResetConfirmRequest, IPasswordResetConfirmRequest
from cvat_sdk.api_client.model.password_reset_serializer_ex_request import PasswordResetSerializerExRequest, IPasswordResetSerializerExRequest
from cvat_sdk.api_client.model.patched_annotation_guide_write_request import PatchedAnnotationGuideWriteRequest, IPatchedAnnotationGuideWriteRequest
from cvat_sdk.api_client.model.patched_cloud_storage_write_request import PatchedCloudStorageWriteRequest, IPatchedCloudStorageWriteRequest
from cvat_sdk.api_client.model.patched_comment_write_request import PatchedCommentWriteRequest, IPatchedCommentWriteRequest
from cvat_sdk.api_client.model.patched_consensus_settings_request import PatchedConsensusSettingsRequest, IPatchedConsensusSettingsRequest
from cvat_sdk.api_client.model.patched_data_meta_write_request import PatchedDataMetaWriteRequest, IPatchedDataMetaWriteRequest
from cvat_sdk.api_client.model.patched_invitation_write_request import PatchedInvitationWriteRequest, IPatchedInvitationWriteRequest
from cvat_sdk.api_client.model.patched_issue_write_request import PatchedIssueWriteRequest, IPatchedIssueWriteRequest
from cvat_sdk.api_client.model.patched_job_data_meta_write_request import PatchedJobDataMetaWriteRequest, IPatchedJobDataMetaWriteRequest
from cvat_sdk.api_client.model.patched_job_validation_layout_write_request import PatchedJobValidationLayoutWriteRequest, IPatchedJobValidationLayoutWriteRequest
from cvat_sdk.api_client.model.patched_job_write_request import PatchedJobWriteRequest, IPatchedJobWriteRequest
from cvat_sdk.api_client.model.patched_label_request import PatchedLabelRequest, IPatchedLabelRequest
from cvat_sdk.api_client.model.patched_labeled_data_request import PatchedLabeledDataRequest, IPatchedLabeledDataRequest
from cvat_sdk.api_client.model.patched_membership_write_request import PatchedMembershipWriteRequest, IPatchedMembershipWriteRequest
from cvat_sdk.api_client.model.patched_organization_write_request import PatchedOrganizationWriteRequest, IPatchedOrganizationWriteRequest
from cvat_sdk.api_client.model.patched_project_write_request import PatchedProjectWriteRequest, IPatchedProjectWriteRequest
from cvat_sdk.api_client.model.patched_project_write_request_target_storage import PatchedProjectWriteRequestTargetStorage
from cvat_sdk.api_client.model.patched_quality_settings_request import PatchedQualitySettingsRequest, IPatchedQualitySettingsRequest
from cvat_sdk.api_client.model.patched_task_validation_layout_write_request import PatchedTaskValidationLayoutWriteRequest, IPatchedTaskValidationLayoutWriteRequest
from cvat_sdk.api_client.model.patched_task_write_request import PatchedTaskWriteRequest, IPatchedTaskWriteRequest
from cvat_sdk.api_client.model.patched_task_write_request_target_storage import PatchedTaskWriteRequestTargetStorage
from cvat_sdk.api_client.model.patched_user_request import PatchedUserRequest, IPatchedUserRequest
from cvat_sdk.api_client.model.patched_webhook_write_request import PatchedWebhookWriteRequest, IPatchedWebhookWriteRequest
from cvat_sdk.api_client.model.plugins import Plugins, IPlugins
from cvat_sdk.api_client.model.project_file_request import ProjectFileRequest, IProjectFileRequest
from cvat_sdk.api_client.model.project_read import ProjectRead, IProjectRead
from cvat_sdk.api_client.model.project_read_target_storage import ProjectReadTargetStorage
from cvat_sdk.api_client.model.project_write_request import ProjectWriteRequest, IProjectWriteRequest
from cvat_sdk.api_client.model.provider_type_enum import ProviderTypeEnum, IProviderTypeEnum
from cvat_sdk.api_client.model.quality_point_size_base import QualityPointSizeBase, IQualityPointSizeBase
from cvat_sdk.api_client.model.quality_report import QualityReport, IQualityReport
from cvat_sdk.api_client.model.quality_report_create_request import QualityReportCreateRequest, IQualityReportCreateRequest
from cvat_sdk.api_client.model.quality_report_jobs_summary import QualityReportJobsSummary, IQualityReportJobsSummary
from cvat_sdk.api_client.model.quality_report_summary import QualityReportSummary, IQualityReportSummary
from cvat_sdk.api_client.model.quality_report_summary_jobs import QualityReportSummaryJobs
from cvat_sdk.api_client.model.quality_report_summary_tasks import QualityReportSummaryTasks
from cvat_sdk.api_client.model.quality_report_target import QualityReportTarget, IQualityReportTarget
from cvat_sdk.api_client.model.quality_report_tasks_summary import QualityReportTasksSummary, IQualityReportTasksSummary
from cvat_sdk.api_client.model.quality_settings import QualitySettings, IQualitySettings
from cvat_sdk.api_client.model.quality_target_metric import QualityTargetMetric, IQualityTargetMetric
from cvat_sdk.api_client.model.register_serializer_ex import RegisterSerializerEx, IRegisterSerializerEx
from cvat_sdk.api_client.model.register_serializer_ex_request import RegisterSerializerExRequest, IRegisterSerializerExRequest
from cvat_sdk.api_client.model.request import Request, IRequest
from cvat_sdk.api_client.model.request_data_operation import RequestDataOperation, IRequestDataOperation
from cvat_sdk.api_client.model.request_owner import RequestOwner
from cvat_sdk.api_client.model.request_status import RequestStatus, IRequestStatus
from cvat_sdk.api_client.model.rest_auth_detail import RestAuthDetail, IRestAuthDetail
from cvat_sdk.api_client.model.role_enum import RoleEnum, IRoleEnum
from cvat_sdk.api_client.model.rq_id import RqId, IRqId
from cvat_sdk.api_client.model.rq_status import RqStatus, IRqStatus
from cvat_sdk.api_client.model.rq_status_state_enum import RqStatusStateEnum, IRqStatusStateEnum
from cvat_sdk.api_client.model.shape_type import ShapeType, IShapeType
from cvat_sdk.api_client.model.signing_request import SigningRequest, ISigningRequest
from cvat_sdk.api_client.model.sorting_method import SortingMethod, ISortingMethod
from cvat_sdk.api_client.model.storage import Storage, IStorage
from cvat_sdk.api_client.model.storage_method import StorageMethod, IStorageMethod
from cvat_sdk.api_client.model.storage_request import StorageRequest, IStorageRequest
from cvat_sdk.api_client.model.storage_type import StorageType, IStorageType
from cvat_sdk.api_client.model.sub_labeled_shape import SubLabeledShape, ISubLabeledShape
from cvat_sdk.api_client.model.sub_labeled_shape_request import SubLabeledShapeRequest, ISubLabeledShapeRequest
from cvat_sdk.api_client.model.sub_labeled_track import SubLabeledTrack, ISubLabeledTrack
from cvat_sdk.api_client.model.sub_labeled_track_request import SubLabeledTrackRequest, ISubLabeledTrackRequest
from cvat_sdk.api_client.model.sublabel import Sublabel, ISublabel
from cvat_sdk.api_client.model.sublabel_mapping_entry_request import SublabelMappingEntryRequest, ISublabelMappingEntryRequest
from cvat_sdk.api_client.model.sublabel_request import SublabelRequest, ISublabelRequest
from cvat_sdk.api_client.model.task_file_request import TaskFileRequest, ITaskFileRequest
from cvat_sdk.api_client.model.task_read import TaskRead, ITaskRead
from cvat_sdk.api_client.model.task_validation_layout_read import TaskValidationLayoutRead, ITaskValidationLayoutRead
from cvat_sdk.api_client.model.task_validation_layout_read_mode import TaskValidationLayoutReadMode
from cvat_sdk.api_client.model.task_write_request import TaskWriteRequest, ITaskWriteRequest
from cvat_sdk.api_client.model.tasks_summary import TasksSummary, ITasksSummary
from cvat_sdk.api_client.model.token import Token, IToken
from cvat_sdk.api_client.model.tracked_shape import TrackedShape, ITrackedShape
from cvat_sdk.api_client.model.tracked_shape_request import TrackedShapeRequest, ITrackedShapeRequest
from cvat_sdk.api_client.model.user import User, IUser
from cvat_sdk.api_client.model.user_identifiers import UserIdentifiers, IUserIdentifiers
from cvat_sdk.api_client.model.validation_mode import ValidationMode, IValidationMode
from cvat_sdk.api_client.model.validation_params_request import ValidationParamsRequest, IValidationParamsRequest
from cvat_sdk.api_client.model.webhook_content_type import WebhookContentType, IWebhookContentType
from cvat_sdk.api_client.model.webhook_delivery_read import WebhookDeliveryRead, IWebhookDeliveryRead
from cvat_sdk.api_client.model.webhook_read import WebhookRead, IWebhookRead
from cvat_sdk.api_client.model.webhook_type import WebhookType, IWebhookType
from cvat_sdk.api_client.model.webhook_write_request import WebhookWriteRequest, IWebhookWriteRequest



# Copyright (C) CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

from abc import ABC
from collections.abc import Sequence
from enum import Enum
from typing import Optional

from cvat_sdk import models
from cvat_sdk.core.proxies.model_proxy import _EntityT


class AnnotationUpdateAction(Enum):
    CREATE = "create"
    UPDATE = "update"
    DELETE = "delete"


class AnnotationCrudMixin(ABC):
    # TODO: refactor

    def get_annotations(self: _EntityT) -> models.ILabeledData:
        (annotations, _) = self.api.retrieve_annotations(getattr(self, self._model_id_field))
        return annotations

    def set_annotations(self: _EntityT, data: models.ILabeledDataRequest):
        self.api.update_annotations(getattr(self, self._model_id_field), labeled_data_request=data)

    def update_annotations(
        self: _EntityT,
        data: models.IPatchedLabeledDataRequest,
        *,
        action: AnnotationUpdateAction = AnnotationUpdateAction.UPDATE,
    ):
        self.api.partial_update_annotations(
            action=action.value,
            id=getattr(self, self._model_id_field),
            patched_labeled_data_request=data,
        )

    def remove_annotations(self: _EntityT, *, ids: Optional[Sequence[int]] = None):
        if ids:
            anns = self.get_annotations()

            if not isinstance(ids, set):
                ids = set(ids)

            anns_to_remove = models.PatchedLabeledDataRequest(
                tags=[models.LabeledImageRequest(**a.to_dict()) for a in anns.tags if a.id in ids],
                tracks=[
                    models.LabeledTrackRequest(**a.to_dict()) for a in anns.tracks if a.id in ids
                ],
                shapes=[
                    models.LabeledShapeRequest(**a.to_dict()) for a in anns.shapes if a.id in ids
                ],
            )

            self.update_annotations(anns_to_remove, action=AnnotationUpdateAction.DELETE)
        else:
            self.api.destroy_annotations(getattr(self, self._model_id_field))



# Copyright (C) 2022 CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

# CVAT REST API
#
# REST API for Computer Vision Annotation Tool (CVAT)  # noqa: E501
#
# The version of the OpenAPI document: 2.40.1
# Contact: support@cvat.ai
# Generated by: https://openapi-generator.tech


from __future__ import annotations

import json
import atexit
import mimetypes
import importlib
from multiprocessing.pool import ThreadPool
import io
import os
import re
import typing
from http.cookies import SimpleCookie
from urllib.parse import quote
from urllib3 import HTTPResponse
from urllib3.fields import RequestField


from cvat_sdk.api_client import rest
from cvat_sdk.api_client.configuration import Configuration
from cvat_sdk.api_client.exceptions import ApiTypeError, ApiValueError, ApiException
from cvat_sdk.api_client.model_utils import (
    ModelNormal,
    ModelSimple,
    ModelComposed,
    check_allowed_values,
    check_validations,
    date,
    datetime,
    deserialize_file,
    file_type,
    model_to_dict,
    none_type,
    validate_and_convert_types,
    to_json,
    get_file_data_and_close_file,
)

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    # Enable introspection. Can't work normally due to cyclic imports
    from cvat_sdk.api_client.apis import *
    from cvat_sdk.api_client.models import *


class ApiClient(object):
    """Generic API client for OpenAPI client library builds.

    OpenAPI generic API client. This client handles the client-
    server communication, and is invariant across implementations. Specifics of
    the methods and models for each application are generated from the OpenAPI
    templates.

    NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech
    Do not edit the class manually.

    Class members:

    assets_api: AssetsApi
    auth_api: AuthApi
    cloudstorages_api: CloudstoragesApi
    comments_api: CommentsApi
    consensus_api: ConsensusApi
    events_api: EventsApi
    guides_api: GuidesApi
    invitations_api: InvitationsApi
    issues_api: IssuesApi
    jobs_api: JobsApi
    labels_api: LabelsApi
    lambda_api: LambdaApi
    memberships_api: MembershipsApi
    organizations_api: OrganizationsApi
    projects_api: ProjectsApi
    quality_api: QualityApi
    requests_api: RequestsApi
    schema_api: SchemaApi
    server_api: ServerApi
    tasks_api: TasksApi
    users_api: UsersApi
    webhooks_api: WebhooksApi
    """

    _pool = None

    def __init__(self,
            configuration: typing.Optional[Configuration] = None,
            headers: typing.Optional[dict[str, str]] = None,
            cookies: typing.Optional[dict[str, str]] = None,
            pool_threads: int = 1):
        """
        :param configuration: configuration object for this client
        :param headers: header to include when making calls to the API
        :param cookies: cookies to include when making calls to the API
        :param pool_threads: The number of threads to use for async requests
            to the API. More threads means more concurrent API requests.
        """

        if configuration is None:
            configuration = Configuration.get_default_copy()
        self.configuration = configuration
        self.pool_threads = pool_threads

        self.rest_client = rest.RESTClientObject(configuration)
        self.default_headers: dict[str, str] = headers or {}
        self.cookies = SimpleCookie()
        if cookies:
            self.cookies.update(cookies)
        # Set default User-Agent.
        self.user_agent = 'cvat_sdk/2.40.1'

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def close(self):
        if self._pool:
            self._pool.close()
            self._pool.join()
            self._pool = None
            if hasattr(atexit, 'unregister'):
                atexit.unregister(self.close)

    @property
    def pool(self):
        """Create thread pool on first request
         avoids instantiating unused threadpool for blocking clients.
        """
        if self._pool is None:
            atexit.register(self.close)
            self._pool = ThreadPool(self.pool_threads)
        return self._pool

    @property
    def user_agent(self):
        """User agent for this API client"""
        return self.default_headers['User-Agent']

    @user_agent.setter
    def user_agent(self, value):
        self.default_headers['User-Agent'] = value

    def _serialize_post_parameter(self, obj):
        if isinstance(obj, (str, int, float, bool)):
            return str(obj)
        elif isinstance(obj, io.IOBase):
            return self._serialize_file(obj)
        raise ApiValueError(
            'Unable to prepare type {} for serialization'.format(
                obj.__class__.__name__))

    def _convert_body_to_post_params(self, body):
        # body must be a flat structure, lists of primitives is possible
        body = self.sanitize_for_serialization(body, read_files=False)
        assert isinstance(body, dict), type(body)

        post_params = []
        for k, v in body.items():
            if isinstance(v, (tuple, list)):
                for i, entry in enumerate(v):
                    post_params.append((
                        f'{k}[{i}]',
                        self._serialize_post_parameter(entry)
                    ))
            else:
                post_params.append((k, self._serialize_post_parameter(v)))
        return post_params

    def set_default_header(self, header_name, header_value):
        self.default_headers[header_name] = header_value

    def __call_api(
        self,
        resource_path: str,
        method: str,
        path_params: typing.Optional[dict[str, typing.Any]] = None,
        query_params: typing.Optional[list[tuple[str, typing.Any]]] = None,
        header_params: typing.Optional[dict[str, typing.Any]] = None,
        body: typing.Optional[typing.Any] = None,
        post_params: typing.Optional[list[tuple[str, typing.Any]]] = None,
        files: typing.Optional[dict[str, list[io.IOBase]]] = None,
        response_schema: typing.Optional[tuple[typing.Any]] = None,
        auth_settings: typing.Optional[list[str]] = None,
        collection_formats: typing.Optional[dict[str, str]] = None,
        *,
        _parse_response: bool = True,
        _request_timeout: typing.Optional[typing.Union[int, float, tuple]] = None,
        _host: typing.Optional[str] = None,
        _check_type: typing.Optional[bool] = None,
        _check_status: bool = True,
        _request_auths: typing.Optional[list[dict[str, typing.Any]]] = None
    ):

        config = self.configuration

        # header parameters
        header_params = header_params or {}
        header_params.update(self.get_common_headers())
        if header_params:
            header_params = self.sanitize_for_serialization(header_params)
            header_params = dict(self.parameters_to_tuples(header_params,
                                                           collection_formats))

        # path parameters
        if path_params:
            path_params = self.sanitize_for_serialization(path_params)
            path_params = self.parameters_to_tuples(path_params,
                                                    collection_formats)
            for k, v in path_params:
                # specified safe chars, encode everything
                resource_path = resource_path.replace(
                    '{%s}' % k,
                    quote(str(v), safe=config.safe_chars_for_path_param)
                )

        # query parameters
        if query_params:
            query_params = self.sanitize_for_serialization(query_params)
            query_params = self.parameters_to_tuples(query_params,
                                                     collection_formats)

        # post parameters
        post_params = post_params if post_params else []
        if post_params or files:
            post_params = self.sanitize_for_serialization(post_params)
            post_params = self.parameters_to_tuples(post_params,
                                                    collection_formats)
            post_params.extend(self.files_parameters(files))

        if header_params.get('Content-Type', '').startswith("multipart"):
            if body:
                post_params.extend(self._convert_body_to_post_params(body))
                body = None

            if post_params:
                post_params = self.parameters_to_multipart(post_params, (dict))
        else:
            # body
            if body:
                body = self.sanitize_for_serialization(body)

        # auth setting
        self.update_params_for_auth(header_params, query_params,
                                    auth_settings, resource_path, method, body,
                                    request_auths=_request_auths)

        # request url
        if _host is None:
            url = self.configuration.host + resource_path
        else:
            # use server/host defined in path or operation instead
            url = _host + resource_path

        try:
            # perform request and return response
            response = self.request(
                method, url, query_params=query_params, headers=header_params,
                post_params=post_params, body=body,
                _parse_response=_parse_response,
                _request_timeout=_request_timeout,
                _check_status=_check_status)
        except ApiException as e:
            e.body = e.body.decode('utf-8')
            raise e

        self.last_response = response

        return_data = None
        if _parse_response:
            self._update_cookies_from_response(response)

            if response_schema:
                return_data = self.deserialize(
                    response,
                    response_schema,
                    _check_type=_check_type
                )

        return (return_data, response)

    def get_common_headers(self) -> dict[str, str]:
        """
        Returns a headers dict with all the required headers for requests
        """

        headers = {}
        headers.update(self.default_headers)
        if self.cookies:
            headers['Cookie'] = self.cookies.output(attrs=[], header="", sep=";").strip()
        return headers

    def _update_cookies_from_response(self, response: HTTPResponse):
        self.cookies.update(SimpleCookie(response.headers.get("Set-Cookie")))

    def parameters_to_multipart(self, params, collection_types):
        """Get parameters as list of tuples, formatting as json if value is collection_types

        :param params: Parameters as list of two-tuples
        :param dict collection_types: Parameter collection types
        :return: Parameters as list of tuple or urllib3.fields.RequestField
        """
        new_params = []
        if collection_types is None:
            collection_types = (dict)
        for k, v in params.items() if isinstance(params, dict) else params:  # noqa: E501
            if isinstance(
                     v, collection_types): # v is instance of collection_type, formatting as application/json
                v = json.dumps(v, ensure_ascii=False).encode("utf-8")
                field = RequestField(k, v)
                field.make_multipart(content_type="application/json; charset=utf-8")
                new_params.append(field)
            else:
                new_params.append((k, v))
        return new_params

    @classmethod
    def sanitize_for_serialization(cls, obj, *,
            read_files: bool = True):
        """Prepares data for transmission before it is sent with the rest client
        If obj is None, return None.
        If obj is str, int, long, float, bool, return directly.
        If obj is datetime.datetime, datetime.date
            convert to string in iso8601 format.
        If obj is list, sanitize each element in the list.
        If obj is dict, return the dict.
        If obj is OpenAPI model, return the properties dict.
        If obj is io.IOBase, return the bytes
        :param obj: The data to serialize.
        :param read_files: Whether to read file data or leave files as is.
        :return: The serialized form of data.
        """
        return to_json(obj, read_files=read_files)

    def deserialize(self, response: HTTPResponse, response_schema: tuple, *, _check_type: bool):
        """Deserializes response into an object.

        :param response (urllib3.HTTPResponse): object to be deserialized.
        :param response_schema: For the response, a tuple containing:
            valid classes
            a list containing valid classes (for list schemas)
            a dict containing a tuple of valid classes as the value
            Example values:
            (str,)
            (Pet,)
            (float, none_type)
            ([int, none_type],)
            ({str: (bool, str, int, float, date, datetime, str, none_type)},)
        :param _check_type (bool): whether to check the types of the data
            received from the server

        :return: deserialized object
        """

        if response_schema == (file_type,):
            # TODO: response schema can be "oneOf" with a file option,
            # this implementation does not cover this.

            # handle file downloading
            # save response body into a tmp file and return the instance
            content_disposition = response.headers.get("Content-Disposition")
            return deserialize_file(response.data, self.configuration,
                                    content_disposition=content_disposition)

        encoding = "utf-8"
        content_type = response.headers.get('content-type')
        if content_type is not None:
            match = re.search(r"charset=([a-zA-Z\-\d]+)[\s\;]?", content_type)
            if match:
                encoding = match.group(1)
        response_data = response.data.decode(encoding)

        # fetch data from response object
        try:
            received_data = json.loads(response_data)
        except ValueError:
            received_data = response_data

        # store our data under the key of 'received_data' so users have some
        # context if they are deserializing a string and the data type is wrong
        deserialized_data = validate_and_convert_types(
            received_data,
            response_schema,
            ['received_data'],
            True,
            _check_type,
            configuration=self.configuration
        )
        return deserialized_data

    def call_api(
        self,
        resource_path: str,
        method: str,
        path_params: typing.Optional[dict[str, typing.Any]] = None,
        query_params: typing.Optional[list[tuple[str, typing.Any]]] = None,
        header_params: typing.Optional[dict[str, typing.Any]] = None,
        body: typing.Optional[typing.Any] = None,
        post_params: typing.Optional[list[tuple[str, typing.Any]]] = None,
        files: typing.Optional[dict[str, list[io.IOBase]]] = None,
        response_schema: typing.Optional[tuple[typing.Any]] = None,
        auth_settings: typing.Optional[list[str]] = None,
        collection_formats: typing.Optional[dict[str, str]] = None,
        *,
        _async_call: typing.Optional[bool] = None,
        _parse_response: bool = True,
        _request_timeout: typing.Optional[typing.Union[int, float, tuple]] = None,
        _host: typing.Optional[str] = None,
        _check_type: typing.Optional[bool] = None,
        _request_auths: typing.Optional[list[dict[str, typing.Any]]] = None,
        _check_status: bool = True,
    ):
        """Makes the HTTP request (synchronous) and returns deserialized data.

        To make an _async_call request, set the _async_call parameter.

        :param resource_path: Path to method endpoint.
        :param method: Method to call.
        :param path_params: Path parameters in the url.
        :param query_params: Query parameters in the url.
        :param header_params: Header parameters to be
            placed in the request header.
        :param body: Request body.
        :param post_params dict: Request post form parameters,
            for `application/x-www-form-urlencoded`, `multipart/form-data`.
        :param auth_settings list: Auth Settings names for the request.
        :param response_schema: For the response, a tuple containing:
            valid classes
            a list containing valid classes (for list schemas)
            a dict containing a tuple of valid classes as the value
            Example values:
            (str,)
            (Pet,)
            (float, none_type)
            ([int, none_type],)
            ({str: (bool, str, int, float, date, datetime, str, none_type)},)
        :param files: key -> field name, value -> a list of open file
            objects for `multipart/form-data`.
        :type files: dict
        :param _async_call bool: execute request asynchronously
        :type _async_call: bool, optional
        :param collection_formats: dict of collection formats for path, query,
            header, and post parameters.
        :type collection_formats: dict, optional
        :param _parse_response: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Response headers will not be
                                 processed (cookies as well). Default is True.
        :type _parse_response: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _check_type: boolean describing if the data back from the server
            should have its type checked.
        :type _check_type: bool, optional
        :param _request_auths: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auths: list, optional
        :return:
            If _async_call parameter is True,
            the request will be called asynchronously.
            The method will return the request thread.
            If parameter _async_call is False or missing,
            then the method will return the response directly.
        """
        params = {
            "resource_path": resource_path,
            "method": method,
            "path_params": path_params,
            "query_params": query_params,
            "header_params": header_params,
            "body": body,
            "post_params": post_params,
            "files": files,
            "response_schema": response_schema,
            "auth_settings": auth_settings,
            "collection_formats": collection_formats,
            "_parse_response": _parse_response,
            "_request_timeout": _request_timeout,
            "_host": _host,
            "_check_type": _check_type,
            "_request_auths": _request_auths,
            "_check_status": _check_status,
        }

        if not _async_call:
            return self.__call_api(**params)

        return self.pool.apply_async(self.__call_api, (), kwds=params)

    def request(self, method, url, query_params=None, headers=None,
                post_params=None, body=None, *, _parse_response=True,
                _request_timeout=None, _check_status=True):
        """Makes the HTTP request using RESTClient."""
        if method == "GET":
            return self.rest_client.GET(url,
                                        query_params=query_params,
                                        _parse_response=_parse_response,
                                        _request_timeout=_request_timeout,
                                        headers=headers,
                                        _check_status=_check_status)
        elif method == "HEAD":
            return self.rest_client.HEAD(url,
                                         query_params=query_params,
                                         _parse_response=_parse_response,
                                         _request_timeout=_request_timeout,
                                         headers=headers,
                                         _check_status=_check_status)
        elif method == "OPTIONS":
            return self.rest_client.OPTIONS(url,
                                            query_params=query_params,
                                            headers=headers,
                                            post_params=post_params,
                                            _parse_response=_parse_response,
                                            _request_timeout=_request_timeout,
                                            _check_status=_check_status,
                                            body=body)
        elif method == "POST":
            return self.rest_client.POST(url,
                                         query_params=query_params,
                                         headers=headers,
                                         post_params=post_params,
                                         _parse_response=_parse_response,
                                         _request_timeout=_request_timeout,
                                         _check_status=_check_status,
                                         body=body)
        elif method == "PUT":
            return self.rest_client.PUT(url,
                                        query_params=query_params,
                                        headers=headers,
                                        post_params=post_params,
                                        _parse_response=_parse_response,
                                        _request_timeout=_request_timeout,
                                        _check_status=_check_status,
                                        body=body)
        elif method == "PATCH":
            return self.rest_client.PATCH(url,
                                          query_params=query_params,
                                          headers=headers,
                                          post_params=post_params,
                                          _parse_response=_parse_response,
                                          _request_timeout=_request_timeout,
                                          _check_status=_check_status,
                                          body=body)
        elif method == "DELETE":
            return self.rest_client.DELETE(url,
                                           query_params=query_params,
                                           headers=headers,
                                           _parse_response=_parse_response,
                                           _request_timeout=_request_timeout,
                                           _check_status=_check_status,
                                           body=body)
        else:
            raise ApiValueError(
                "http method must be `GET`, `HEAD`, `OPTIONS`,"
                " `POST`, `PATCH`, `PUT` or `DELETE`."
            )

    def parameters_to_tuples(self, params, collection_formats):
        """Get parameters as list of tuples, formatting collections.

        :param params: Parameters as dict or list of two-tuples
        :param dict collection_formats: Parameter collection formats
        :return: Parameters as list of tuples, collections formatted
        """
        new_params = []
        if collection_formats is None:
            collection_formats = {}
        for k, v in params.items() if isinstance(params, dict) else params:  # noqa: E501
            if k in collection_formats:
                collection_format = collection_formats[k]
                if collection_format == 'multi':
                    new_params.extend((k, value) for value in v)
                else:
                    if collection_format == 'ssv':
                        delimiter = ' '
                    elif collection_format == 'tsv':
                        delimiter = '\t'
                    elif collection_format == 'pipes':
                        delimiter = '|'
                    else:  # csv is the default
                        delimiter = ','
                    new_params.append(
                        (k, delimiter.join(str(value) for value in v)))
            else:
                new_params.append((k, v))
        return new_params

    def _serialize_file(self, file_instance: io.IOBase) -> tuple[str, typing.Union[str, bytes], str]:
        if file_instance.closed is True:
            raise ApiValueError("Cannot read a closed file.")
        filename = os.path.basename(file_instance.name)

        filedata = get_file_data_and_close_file(file_instance)

        mimetype = mimetypes.guess_type(filename)[0] or 'application/octet-stream'

        return filename, filedata, mimetype

    def files_parameters(self,
                         files: typing.Optional[dict[str, list[io.IOBase]]] = None):
        """Builds form parameters.

        :param files: None or a dict with key=param_name and
            value is a list of open file objects
        :return: List of tuples of form parameters with file data
        """
        if files is None:
            return []

        params = []
        for param_name, file_instances in files.items():
            if file_instances is None:
                # if the file field is nullable, skip None values
                continue
            for file_instance in file_instances:
                if file_instance is None:
                    # if the file field is nullable, skip None values
                    continue

                try:
                    params.append((param_name, self._serialize_file(file_instance)))
                except ApiValueError as e:
                    raise ApiValueError("The passed in file_type "
                        "for %s must be open." % param_name) from e

        return params

    def select_header_accept(self, accepts):
        """Returns `Accept` based on an array of accepts provided.

        :param accepts: List of headers.
        :return: Accept (e.g. application/json).
        """
        if not accepts:
            return

        accepts = [x.lower() for x in accepts]

        if 'application/json' in accepts:
            return 'application/json'
        else:
            return ', '.join(accepts)

    def select_header_content_type(self, content_types, method=None, body=None):
        """Returns `Content-Type` based on an array of content_types provided.

        :param content_types: List of content-types.
        :param method: http method (e.g. POST, PATCH).
        :param body: http body to send.
        :return: Content-Type (e.g. application/json).
        """
        if not content_types:
            return None

        content_types = [x.lower() for x in content_types]

        if (method == 'PATCH' and
                'application/json-patch+json' in content_types and
                isinstance(body, list)):
            return 'application/json-patch+json'

        if 'application/json' in content_types or '*/*' in content_types:
            return 'application/json'
        else:
            return content_types[0]

    def update_params_for_auth(self, headers, queries, auth_settings,
                               resource_path, method, body, request_auths=None):
        """Updates header and query params based on authentication setting.

        :param headers: Header parameters dict to be updated.
        :param queries: Query parameters tuple list to be updated.
        :param auth_settings: Authentication setting identifiers list.
        :param resource_path: A string representation of the HTTP request resource path.
        :param method: A string representation of the HTTP request method.
        :param body: A object representing the body of the HTTP request.
            The object type is the return value of _encoder.default().
        :param request_auths: if set, the provided settings will
            override the token in the configuration.
        """
        if not auth_settings:
            return

        if request_auths:
            for auth_setting in request_auths:
                self._apply_auth_params(
                    headers, queries, resource_path, method, body, auth_setting)
            return

        for auth in auth_settings:
            auth_setting = self.configuration.auth_settings().get(auth)
            if auth_setting:
                self._apply_auth_params(
                    headers, queries, resource_path, method, body, auth_setting)

    def _apply_auth_params(self, headers, queries, resource_path, method, body, auth_setting):
        if auth_setting['in'] == 'cookie':
            headers['Cookie'] = auth_setting['key'] + "=" + auth_setting['value']
        elif auth_setting['in'] == 'header':
            if auth_setting['type'] != 'http-signature':
                headers[auth_setting['key']] = auth_setting['value']
        elif auth_setting['in'] == 'query':
            queries.append((auth_setting['key'], auth_setting['value']))
        else:
            raise ApiValueError(
                'Authentication token must be in `query` or `header`'
            )


    assets_api: 'AssetsApi'
    auth_api: 'AuthApi'
    cloudstorages_api: 'CloudstoragesApi'
    comments_api: 'CommentsApi'
    consensus_api: 'ConsensusApi'
    events_api: 'EventsApi'
    guides_api: 'GuidesApi'
    invitations_api: 'InvitationsApi'
    issues_api: 'IssuesApi'
    jobs_api: 'JobsApi'
    labels_api: 'LabelsApi'
    lambda_api: 'LambdaApi'
    memberships_api: 'MembershipsApi'
    organizations_api: 'OrganizationsApi'
    projects_api: 'ProjectsApi'
    quality_api: 'QualityApi'
    requests_api: 'RequestsApi'
    schema_api: 'SchemaApi'
    server_api: 'ServerApi'
    tasks_api: 'TasksApi'
    users_api: 'UsersApi'
    webhooks_api: 'WebhooksApi'

    _apis: dict[str, object] = { 
        'assets_api': [None, 'AssetsApi'],
        'auth_api': [None, 'AuthApi'],
        'cloudstorages_api': [None, 'CloudstoragesApi'],
        'comments_api': [None, 'CommentsApi'],
        'consensus_api': [None, 'ConsensusApi'],
        'events_api': [None, 'EventsApi'],
        'guides_api': [None, 'GuidesApi'],
        'invitations_api': [None, 'InvitationsApi'],
        'issues_api': [None, 'IssuesApi'],
        'jobs_api': [None, 'JobsApi'],
        'labels_api': [None, 'LabelsApi'],
        'lambda_api': [None, 'LambdaApi'],
        'memberships_api': [None, 'MembershipsApi'],
        'organizations_api': [None, 'OrganizationsApi'],
        'projects_api': [None, 'ProjectsApi'],
        'quality_api': [None, 'QualityApi'],
        'requests_api': [None, 'RequestsApi'],
        'schema_api': [None, 'SchemaApi'],
        'server_api': [None, 'ServerApi'],
        'tasks_api': [None, 'TasksApi'],
        'users_api': [None, 'UsersApi'],
        'webhooks_api': [None, 'WebhooksApi'],
    }

    def _make_api_instance(self, klass_name):
        package = __name__.rsplit('.', maxsplit=1)[0]
        module = importlib.import_module(package + '.apis')
        api_klass = getattr(module, klass_name)
        return api_klass(self)

    def __getattr__(self, key):
        notfound = object()
        api_instance, api_klassname = self._apis.get(key, notfound)
        if api_instance is notfound:
            raise AttributeError(f"Can't find the '{key}' attribute")

        if api_instance is None:
            api_instance = self._make_api_instance(api_klassname)
            setattr(self, key, api_instance)

        return api_instance


class Endpoint(object):
    def __init__(self,
            settings: typing.Optional[dict[str, typing.Any]] = None,
            params_map: typing.Optional[dict[str, typing.Any]] = None,
            root_map: typing.Optional[dict[str, typing.Any]] = None,
            headers_map: typing.Optional[dict[str, typing.Any]] = None,
            api_client: typing.Optional[ApiClient] = None
    ):
        """Creates an endpoint

        Args:
            settings (dict): see below key value pairs
                'response_schema' (tuple/None): response type
                'auth' (list): a list of auth type keys
                'endpoint_path' (str): the endpoint path
                'operation_id' (str): endpoint string identifier
                'http_method' (str): POST/PUT/PATCH/GET etc
                'servers' (list): list of str servers that this endpoint is at
            params_map (dict): see below key value pairs
                'all' (list): list of str endpoint parameter names
                'required' (list): list of required parameter names
                'nullable' (list): list of nullable parameter names
                'enum' (list): list of parameters with enum values
                'validation' (list): list of parameters with validations
            root_map (dict):
                'validations' (dict): the dict mapping endpoint parameter tuple
                    paths to their validation dictionaries
                'allowed_values' (dict): the dict mapping endpoint parameter
                    tuple paths to their allowed_values (enum) dictionaries
                'openapi_types' (dict): param_name to openapi type
                'attribute_map' (dict): param_name to camelCase name
                'location_map' (dict): param_name to  'body', 'file', 'form',
                    'header', 'path', 'query'
                collection_format_map (dict): param_name to `csv` etc.
            headers_map (dict): see below key value pairs
                'accept' (list): list of Accept header strings
                'content_type' (list): list of Content-Type header strings
            api_client (ApiClient) api client instance
        """

        self.settings = settings
        self.params_map = params_map
        self.params_map['all'].extend([
            '_async_call',
            '_host_index',
            '_parse_response',
            '_request_timeout',
            '_validate_inputs',
            '_validate_outputs',
            '_check_status',
            '_content_type',
            '_spec_property_naming',
            '_request_auths'
        ])
        self.params_map['nullable'].extend(['_request_timeout'])
        self.validations = root_map['validations']
        self.allowed_values = root_map['allowed_values']
        self.openapi_types = root_map['openapi_types']
        extra_types = {
            '_async_call': (bool,),
            '_host_index': (none_type, int),
            '_parse_response': (bool,),
            '_request_timeout': (none_type, float, (float,), [float], int, (int,), [int]),
            '_validate_inputs': (bool,),
            '_validate_outputs': (bool,),
            '_check_status': (bool,),
            '_spec_property_naming': (bool,),
            '_content_type': (none_type, str),
            '_request_auths': (none_type, list)
        }
        self.openapi_types.update(extra_types)
        self.attribute_map = root_map['attribute_map']
        self.location_map = root_map['location_map']
        self.collection_format_map = root_map['collection_format_map']
        self.headers_map = headers_map
        self.api_client = api_client

    @property
    def path(self) -> str:
        return self.settings['endpoint_path']

    def __validate_inputs(self, kwargs):
        for param in self.params_map['enum']:
            if param in kwargs:
                check_allowed_values(
                    self.allowed_values,
                    (param,),
                    kwargs[param]
                )

        for param in self.params_map['validation']:
            if param in kwargs:
                check_validations(
                    self.validations,
                    (param,),
                    kwargs[param],
                    configuration=self.api_client.configuration
                )

        if kwargs['_validate_inputs'] is False:
            return

        for key, value in kwargs.items():
            fixed_val = validate_and_convert_types(
                value,
                self.openapi_types[key],
                [key],
                kwargs['_spec_property_naming'],
                kwargs['_validate_inputs'],
                configuration=self.api_client.configuration
            )
            kwargs[key] = fixed_val

    def __gather_params(self, kwargs):
        params = {
            'body': None,
            'collection_format': {},
            'file': {},
            'form': [],
            'header': {},
            'path': {},
            'query': []
        }

        for param_name, param_value in kwargs.items():
            param_location = self.location_map.get(param_name)
            if param_location is None:
                continue
            if param_location:
                if param_location == 'body':
                    params['body'] = param_value
                    continue
                base_name = self.attribute_map[param_name]
                if (param_location == 'form' and
                        self.openapi_types[param_name] == (file_type,)):
                    params['file'][base_name] = [param_value]
                elif (param_location == 'form' and
                        self.openapi_types[param_name] == ([file_type],)):
                    # param_value is already a list
                    params['file'][base_name] = param_value
                elif param_location in {'form', 'query'}:
                    param_value_full = (base_name, param_value)
                    params[param_location].append(param_value_full)
                if param_location not in {'form', 'query'}:
                    params[param_location][base_name] = param_value
                collection_format = self.collection_format_map.get(param_name)
                if collection_format:
                    params['collection_format'][base_name] = collection_format

        return params

    def call_with_http_info(self,
            _parse_response: bool = True,
            _request_timeout: typing.Union[int, float, tuple] = None,
            _validate_inputs: bool = True,
            _validate_outputs: bool = True,
            _check_status: bool = True,
            _spec_property_naming: bool = False,
            _content_type: typing.Optional[str] = None,
            _host_index: typing.Optional[int] = None,
            _request_auths: typing.Optional[list] = None,
            _async_call: bool = False,
            **kwargs) -> tuple[typing.Optional[typing.Any], HTTPResponse]:
        """
        Keyword Args:
            endpoint args
            _parse_response (bool): if False, the response data will not be parsed,
                None is returned for data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _validate_inputs (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _validate_outputs (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _check_status (bool): whether to check response status
                for being positive or not.
                Default is True
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            _async_call (bool): execute request asynchronously

        Returns:
            (parsed_model, response)
                If the method is called asynchronously, returns the request
                thread.
        """

        kwargs['_parse_response'] = _parse_response
        kwargs['_request_timeout'] = _request_timeout
        kwargs['_validate_inputs'] = _validate_inputs
        kwargs['_validate_outputs'] = _validate_outputs
        kwargs['_check_status'] = _check_status
        kwargs['_spec_property_naming'] = _spec_property_naming
        kwargs['_content_type'] = _content_type
        kwargs['_host_index'] = _host_index
        kwargs['_request_auths'] = _request_auths
        kwargs['_async_call'] = _async_call

        try:
            index = self.api_client.configuration.server_operation_index.get(
                self.settings['operation_id'], self.api_client.configuration.server_index
            ) if kwargs['_host_index'] is None else kwargs['_host_index']
            server_variables = self.api_client.configuration.server_operation_variables.get(
                self.settings['operation_id'], self.api_client.configuration.server_variables
            )
            _host = self.api_client.configuration.get_host_from_settings(
                index, variables=server_variables, servers=self.settings['servers']
            )
        except IndexError:
            if self.settings['servers']:
                raise ApiValueError(
                    "Invalid host index. Must be 0 <= index < %s" %
                    len(self.settings['servers'])
                )
            _host = None

        for key, value in kwargs.items():
            if key not in self.params_map['all']:
                raise ApiTypeError(
                    "Got an unexpected parameter '%s'"
                    " to method `%s`" %
                    (key, self.settings['operation_id'])
                )
            # only throw this nullable ApiValueError if _validate_inputs
            # is False, if _validate_inputs==True we catch this case
            # in self.__validate_inputs
            if (key not in self.params_map['nullable'] and value is None
                    and kwargs['_validate_inputs'] is False):
                raise ApiValueError(
                    "Value may not be None for non-nullable parameter `%s`"
                    " when calling `%s`" %
                    (key, self.settings['operation_id'])
                )

        for key in self.params_map['required']:
            if key not in kwargs.keys():
                raise ApiValueError(
                    "Missing the required parameter `%s` when calling "
                    "`%s`" % (key, self.settings['operation_id'])
                )

        self.__validate_inputs(kwargs)

        params = self.__gather_params(kwargs)

        accept_headers_list = self.headers_map['accept']
        if accept_headers_list:
            params['header']['Accept'] = self.api_client.select_header_accept(
                accept_headers_list)

        if kwargs.get('_content_type'):
            params['header']['Content-Type'] = kwargs['_content_type']
        else:
            content_type_headers_list = self.headers_map['content_type']
            if content_type_headers_list:
                if params['body'] != "":
                    content_types_list = self.api_client.select_header_content_type(
                        content_type_headers_list, self.settings['http_method'],
                        params['body'])
                    if content_types_list:
                        params['header']['Content-Type'] = content_types_list

        return self.api_client.call_api(
            self.settings['endpoint_path'], self.settings['http_method'],
            params['path'],
            params['query'],
            params['header'],
            body=params['body'],
            post_params=params['form'],
            files=params['file'],
            response_schema=self.settings['response_schema'],
            auth_settings=self.settings['auth'],
            _async_call=kwargs['_async_call'],
            _check_type=kwargs['_validate_outputs'],
            _check_status=kwargs['_check_status'],
            _parse_response=kwargs['_parse_response'],
            _request_timeout=kwargs['_request_timeout'],
            _host=_host,
            _request_auths=kwargs['_request_auths'],
            collection_formats=params['collection_format'])



# Copyright (C) CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

from __future__ import annotations

from cvat_sdk.api_client import apis, models
from cvat_sdk.core.proxies.model_proxy import (
    ModelCreateMixin,
    ModelDeleteMixin,
    ModelListMixin,
    ModelRetrieveMixin,
    ModelUpdateMixin,
    build_model_bases,
)

_OrganizationEntityBase, _OrganizationRepoBase = build_model_bases(
    models.OrganizationRead, apis.OrganizationsApi, api_member_name="organizations_api"
)


class Organization(
    models.IOrganizationRead,
    _OrganizationEntityBase,
    ModelUpdateMixin[models.IPatchedOrganizationWriteRequest],
    ModelDeleteMixin,
):
    _model_partial_update_arg = "patched_organization_write_request"


class OrganizationsRepo(
    _OrganizationRepoBase,
    ModelCreateMixin[Organization, models.IOrganizationWriteRequest],
    ModelListMixin[Organization],
    ModelRetrieveMixin[Organization],
):
    _entity_type = Organization



# Copyright (C) CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

import abc
from enum import Enum, auto
from typing import Optional

import attrs
import attrs.validators
import PIL.Image

import cvat_sdk.core
import cvat_sdk.core.exceptions
import cvat_sdk.models as models


class UnsupportedDatasetError(cvat_sdk.core.exceptions.CvatSdkException):
    pass


@attrs.frozen
class FrameAnnotations:
    """
    Contains annotations that pertain to a single frame.
    """

    tags: list[models.LabeledImage] = attrs.Factory(list)
    shapes: list[models.LabeledShape] = attrs.Factory(list)


class MediaElement(metaclass=abc.ABCMeta):
    """
    The media part of a dataset sample.
    """

    @abc.abstractmethod
    def load_image(self) -> PIL.Image.Image:
        """
        Loads the media data and returns it as a PIL Image object.
        """
        ...


@attrs.frozen
class Sample:
    """
    Represents an element of a dataset.
    """

    frame_index: int
    """Index of the corresponding frame in its task."""

    frame_name: str
    """File name of the frame in its task."""

    annotations: Optional[FrameAnnotations]
    """
    Annotations belonging to the frame.

    Will be None if the dataset was created without loading annotations.
    """

    media: MediaElement
    """Media data of the frame."""


class MediaDownloadPolicy(Enum):
    """Defines policies controlling when media data is downloaded."""

    PRELOAD_ALL = auto()
    """Download and cache all media data when the dataset object is created."""

    FETCH_FRAMES_ON_DEMAND = auto()
    """Download the media element for each frame whenever MediaElement.load_* is invoked."""



# Copyright (C) CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

import PIL.Image

import cvat_sdk.auto_annotation as cvataa
import cvat_sdk.models as models

from ._torchvision import TorchvisionFunction


class _TorchvisionDetectionFunction(TorchvisionFunction):
    _label_type = "rectangle"

    def detect(
        self, context: cvataa.DetectionFunctionContext, image: PIL.Image.Image
    ) -> list[models.LabeledShapeRequest]:
        conf_threshold = context.conf_threshold or 0
        results = self._model([self._transforms(image)])

        return [
            cvataa.rectangle(label.item(), [x.item() for x in box])
            for result in results
            for box, label, score in zip(result["boxes"], result["labels"], result["scores"])
            if score >= conf_threshold
        ]


create = _TorchvisionDetectionFunction



# Copyright (C) 2022 CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

# CVAT REST API
#
# REST API for Computer Vision Annotation Tool (CVAT)  # noqa: E501
#
# The version of the OpenAPI document: 2.40.1
# Contact: support@cvat.ai
# Generated by: https://openapi-generator.tech


__version__ = "2.40.1"

from cvat_sdk.api_client.api_client import ApiClient

from cvat_sdk.api_client.configuration import Configuration

from cvat_sdk.api_client.exceptions import OpenApiException
from cvat_sdk.api_client.exceptions import ApiAttributeError
from cvat_sdk.api_client.exceptions import ApiTypeError
from cvat_sdk.api_client.exceptions import ApiValueError
from cvat_sdk.api_client.exceptions import ApiKeyError
from cvat_sdk.api_client.exceptions import ApiException




# Copyright (C) 2022 CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

# CVAT REST API
#
# REST API for Computer Vision Annotation Tool (CVAT)  # noqa: E501
#
# The version of the OpenAPI document: 2.40.1
# Contact: support@cvat.ai
# Generated by: https://openapi-generator.tech


# Import all APIs into this package.
# If you have many APIs here with many many models used in each API this may
# raise a `RecursionError`.
# In order to avoid this, import only the API that you directly need like:
#
#   from cvat_sdk.api_client.api.assets_api import AssetsApi
#
# or import this package, but before doing it, use:
#
#   import sys
#   sys.setrecursionlimit(n)

# Import APIs into API package:
from cvat_sdk.api_client.api.assets_api import AssetsApi
from cvat_sdk.api_client.api.auth_api import AuthApi
from cvat_sdk.api_client.api.cloudstorages_api import CloudstoragesApi
from cvat_sdk.api_client.api.comments_api import CommentsApi
from cvat_sdk.api_client.api.consensus_api import ConsensusApi
from cvat_sdk.api_client.api.events_api import EventsApi
from cvat_sdk.api_client.api.guides_api import GuidesApi
from cvat_sdk.api_client.api.invitations_api import InvitationsApi
from cvat_sdk.api_client.api.issues_api import IssuesApi
from cvat_sdk.api_client.api.jobs_api import JobsApi
from cvat_sdk.api_client.api.labels_api import LabelsApi
from cvat_sdk.api_client.api.lambda_api import LambdaApi
from cvat_sdk.api_client.api.memberships_api import MembershipsApi
from cvat_sdk.api_client.api.organizations_api import OrganizationsApi
from cvat_sdk.api_client.api.projects_api import ProjectsApi
from cvat_sdk.api_client.api.quality_api import QualityApi
from cvat_sdk.api_client.api.requests_api import RequestsApi
from cvat_sdk.api_client.api.schema_api import SchemaApi
from cvat_sdk.api_client.api.server_api import ServerApi
from cvat_sdk.api_client.api.tasks_api import TasksApi
from cvat_sdk.api_client.api.users_api import UsersApi
from cvat_sdk.api_client.api.webhooks_api import WebhooksApi



# Copyright (C) CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

import math
from collections.abc import Iterator

import numpy as np
import PIL.Image
from skimage import measure
from torch import Tensor

import cvat_sdk.auto_annotation as cvataa
import cvat_sdk.models as models
from cvat_sdk.masks import encode_mask

from ._torchvision import TorchvisionFunction


def _is_positively_oriented(contour: np.ndarray) -> bool:
    ys, xs = contour.T

    # This is the shoelace formula, except we only need the sign of the result,
    # so we compare instead of subtracting. Compared to the typical formula,
    # the sign is inverted, because the Y axis points downwards.
    return np.sum(xs * np.roll(ys, -1)) < np.sum(ys * np.roll(xs, -1))


def _generate_shapes(
    context: cvataa.DetectionFunctionContext, box: Tensor, mask: Tensor, label: Tensor
) -> Iterator[models.LabeledShapeRequest]:
    LEVEL = 0.5

    if context.conv_mask_to_poly:
        # Since we treat mask values of exactly LEVEL as true, we'd like them
        # to also be considered high by find_contours. And for that, the level
        # parameter must be slightly less than LEVEL.
        contours = measure.find_contours(mask[0].detach().numpy(), level=math.nextafter(LEVEL, 0))

        for contour in contours:
            if len(contour) < 3 or _is_positively_oriented(contour):
                continue

            contour = measure.approximate_polygon(contour, tolerance=2.5)

            yield cvataa.polygon(label.item(), contour[:, ::-1].ravel().tolist())

    else:
        yield cvataa.mask(label.item(), encode_mask(mask[0] >= LEVEL, box.tolist()))


class _TorchvisionInstanceSegmentationFunction(TorchvisionFunction):
    _label_type = "mask"

    def detect(
        self, context: cvataa.DetectionFunctionContext, image: PIL.Image.Image
    ) -> list[models.LabeledShapeRequest]:
        conf_threshold = context.conf_threshold or 0
        results = self._model([self._transforms(image)])

        return [
            shape
            for result in results
            for box, mask, label, score in zip(
                result["boxes"], result["masks"], result["labels"], result["scores"]
            )
            if score >= conf_threshold
            for shape in _generate_shapes(context, box, mask, label)
        ]


create = _TorchvisionInstanceSegmentationFunction
